# OpenXPKI Enrollment Interface

This is a certificate enrollment interface for OpenXPKI. Basically, it
runs on a bastion host and accepts CSRs from external users. These
CSRs are passed to an internal OpenXPKI daemon via SCEP using the sscep
to forward the request.

# REQUIREMENTS

* web UI is written in Perl using Mojolicious framework
* served using Mojolicious-supported systems (e.g. Apache/CGI, etc)
* incoming request may be authenticated externally
* accessing the URI '/' lists all possible operations
* POST to the URI '/upload' receives the CSR
* CSR is forwarded to SCEP server using sscep wrapper script
* The call to SSCEP must be properly escaped to prevent tainted information
* web UI may be forward-facing to Internet, so security is crucial

# ARCHITECTURE

	+------------------------------+
	| Apache HTTP Server           |
	+------------------------------+
			|
			| (CGI call of 'enroller' script)
			|
			V
	+------------------------------+
	| Enroller Web UI              |
	+------------------------------+
			|
			| (enroller calls wrapper script)
			|
			V
	+------------------------------+
	| sscep Wrapper Script         |
	+------------------------------+
			|
			| (wrapper script calls sscep client)
			|
			V
	+------------------------------+
	| sscep client                 |
	+------------------------------+
			|
			| (SCEP request sent to server)
			|
			V
	+------------------------------+
	| SCEP Server                  |
	+------------------------------+

## Apache HTTP Server

One method of serving the Enrollment UI is via Apache/CGI using the ScriptAlias directive:

    <Directory /srv/www/enroller>
		Options -FollowSymLinks
		AllowOverride None
		Order allow,deny
		Allow from all
	</Directory>
	ScriptAlias / /srv/www/enroller/script/enroller/
    

## SETUP

For this to run correctly, the sscep program needs the configuration file
etc/sscep.conf. See the example etc/sscep.conf.example for details.

### Customization

The Enroller::Upload::upload() method calls a wrapper command to forward
the CSR to the CA for certificate issuance. To prevent tainting, the 
system() call is used. The command name and the filename for the local
copy of the CSR are passed to the system() call as a list to prevent shell
expansion. The filename must be locally generated by the PSGI script -- do
NOT accept tainted data from the user.

The environment variable ENROLL\_FORWARD\_CMD may contain the name of the
command used to forward the CSR.

The default forwarder is script/sscep-wrapper.

## RUNNING

If you run the script in a PSGI or CGI environment of a web server, it should
just "do the right thing(tm)". To run a test daemon that is reachable via 
your web browser, run the following:

    script/enroller daemon

To run the test cases, use the following:

    script/enroller test

